#!/usr/bin/env bash
# run test model 10x using various numerical methods
MODEL=./ctest
RESULTFILE=ctest-results-`date +"%Y-%m-%d"`.log
make

if [ ! -x $MODEL ]; then
    echo "Compilation problem"
    exit
fi

(
echo "=============================="
date
uname -a
cat /proc/cpuinfo | head -8
echo "=============================="
) >>$RESULTFILE

function bench() {
  METHOD="$1"
  echo "$METHOD"
  echo -n "$METHOD:	"             >>$RESULTFILE
  for i in 1 2 3 4 5 6 7 8 9 0; do
    (time -p $MODEL $METHOD >/dev/null) 2>&1
    # mv ctest.dat ctest-$1.dat
    rm -f ctest.dat # remove simulation results
  done | 
         grep 'user' |
         sed 's/user[ 	]*//' |
         sort -n |
         head -8 |
         tail -n +3 |
         awk '
### compute average, min, max
BEGIN{ min=1e10; max=0; }
{ if($1!=0) {
    x = $1 + 0.0
    sum += x
    if(max<x) max=x
    if(min>x) min=x
    n++
  }
}
END { printf("avg=%.2f n=%d  min=%.3f  max=%.3f \n", sum/n, n, min, max); }

' >>$RESULTFILE
}

bench euler
bench rke
bench rkf8
bench abm4

(
date
echo
echo
) >>$RESULTFILE

echo
